SET SERVEROUTPUT ON

/*---------------------------------------------------------------------------------------
							PROCEDURE p_insertaLog
---------------------------------------------------------------------------------------*/

CREATE OR REPLACE PROCEDURE p_insertaLog (
nomproc IN Varchar2,
f_ini IN DATE,
f_fin IN DATE,
paramsent IN Varchar2,
paramssal IN Varchar2
) IS
BEGIN
	INSERT INTO LOGS (nombreproc, fechainicio, fechafin, paramsentrada, paramssalida)
	VALUES (nomproc, f_ini, f_fin, paramsent, paramssal);
	
	COMMIT;	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		DBMS_OUTPUT.PUT_LINE('ERROR: Error al almacenar el log.');
	
	WHEN OTHERS THEN
	
		DBMS_OUTPUT.PUT_LINE('ERROR: Error al insertar el log: ' || sqlerrm);
	
END;
/

CREATE OR REPLACE PROCEDURE p_altaCentro (
v_nom IN varchar2,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaCentro';

BEGIN

 	params := v_nom;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (v_nom IS NULL OR v_nom = '') THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO CENTROS (nombre)
	VALUES (v_nom);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El centro ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un centro: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/



CREATE OR REPLACE PROCEDURE p_altaAlumno (
v_nom IN Varchar2,
v_ape IN Varchar2,
v_dni IN Varchar2,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaAlumno';

BEGIN

 	params := v_nom || ',' || v_ape || ',' || v_dni;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (v_nom IS NULL OR v_nom = '') OR
	(v_ape IS NULL OR v_ape = '') OR 
	(v_dni IS NULL OR v_dni = '') THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO ALUMNOS (nombre, apellidos, dni)
	VALUES (v_nom, v_ape, v_dni);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El alumno ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el alumno';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un alumno: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/





CREATE OR REPLACE PROCEDURE p_bajaAlumno (
id IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaAlumno';

BEGIN

 	params := id;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE ALUMNOS WHERE idalumno = id;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
   END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: El alumno no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un alumno: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modAlumno (
id IN NUMBER,
RSP OUT Varchar2,
v_nom IN Varchar2 DEFAULT '',
v_ape IN Varchar2 DEFAULT '',
v_dni IN Varchar2 DEFAULT ''
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modAlumno';

BEGIN

 	params := id || ',' || v_nom || ',' || v_ape || ',' || v_dni;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	
	UPDATE ALUMNOS SET nombre = DECODE(v_nom, '', NOMBRE, NULL, NOMBRE, v_nom),
		   		   	   apellidos = DECODE(v_ape, '', APELLIDOS, NULL, APELLIDOS, v_ape),
					   dni = DECODE(v_dni, '', DNI, NULL, DNI, v_dni)
					   WHERE idAlumno = id;	 

	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;				   
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el alumno';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El alumno no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar un alumno: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_altaAlumnoCurso (
id_curso IN number,
id_alumno IN number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaAlumnoCurso';

BEGIN

 	params := id_curso || ',' || id_alumno;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR
	   (id_alumno IS NULL OR id_alumno = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO ALUMNOSCURSO (idalumno, idcurso)
	VALUES (id_alumno, id_curso);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El alumno ya est inscrito en este curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el alumno en este curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un alumno en un curso: '  || sqlerrm ;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
END;
/

CREATE OR REPLACE PROCEDURE p_bajaAlumnoCurso (
id_curso IN NUMBER,
id_alumno IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaAlumnoCurso';

BEGIN

 	params := id_curso || ',' || id_alumno;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR 
	   (id_alumno IS NULL OR id_alumno = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE ALUMNOSCURSO WHERE idcurso = id_curso and idalumno = id_alumno;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: El alumno no cursa este curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un alumno en este curso: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/




CREATE OR REPLACE PROCEDURE p_altaProfesor (
v_nom IN Varchar2,
v_ape IN Varchar2,
v_dni IN Varchar2,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaProfesor';

BEGIN

 	params := v_nom || ',' || v_ape || ',' || v_dni;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (v_nom IS NULL OR v_nom = '') OR
	(v_ape IS NULL OR v_ape = '') OR 
	(v_dni IS NULL OR v_dni = '') THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO PROFESORES (nombre, apellidos, dni)
	VALUES (v_nom, v_ape, v_dni);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El profesor ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el profesor';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un profesor: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_bajaProfesor (
id IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaProfesor';

BEGIN

 	params := id;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE PROFESORES WHERE idprofesor = id;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
   END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: El profesor no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un profesor: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modProfesor (
id IN NUMBER,
RSP OUT Varchar2,
v_nom IN Varchar2 DEFAULT '',
v_ape IN Varchar2 DEFAULT '',
v_dni IN Varchar2 DEFAULT ''
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modProfesor';

BEGIN

 	params := id || ',' || v_nom || ',' || v_ape || ',' || v_dni;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE PROFESORES SET nombre = DECODE(v_nom, '', NOMBRE, NULL, NOMBRE, v_nom),
		   		   	   apellidos = DECODE(v_ape, '', APELLIDOS, NULL, APELLIDOS, v_ape),
					   dni = DECODE(v_dni, '', DNI, NULL, DNI, v_dni)
					   WHERE idprofesor = id;	 

   	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el profesor';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El profesor no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar un profesor: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaCurso (
v_nom IN varchar2,
f_inicio IN date,
f_fin IN date,
id_centro number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
f_aux DATE;
id_curso NUMBER;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaCurso';

BEGIN

 	params := v_nom || ',' || f_inicio || ',' || f_fin || ',' || id_centro;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (v_nom IS NULL OR v_nom = '') OR
	   (f_inicio IS NULL OR f_inicio = '') OR
	   (f_fin IS NULL OR f_fin = '') OR
	   (id_centro IS NULL OR id_centro = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO CURSOS (nombre, fechainicio, fechafin, idcentro)
	VALUES (v_nom, f_inicio, f_fin, id_centro);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El curso ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un curso: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_bajaCurso (
id IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaCurso';

BEGIN

 	params := id;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE CURSOS WHERE idcurso = id;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
   
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: El curso no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un curso: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/



CREATE OR REPLACE PROCEDURE p_modCurso (
id IN NUMBER,
RSP OUT Varchar2,
v_nom IN Varchar2 DEFAULT '',
f_inicio IN date DEFAULT '',
f_fin IN date DEFAULT '',
id_centro number DEFAULT 0
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(12) := 'p_modCurso';

BEGIN

 	params := id || ',' || v_nom || ',' || f_inicio || ',' || f_fin || ',' || id_centro;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE CURSOS SET nombre = DECODE(v_nom, '', NOMBRE, NULL, NOMBRE, v_nom),
		   		   	   fechainicio = DECODE(f_inicio, '', FECHAINICIO, NULL, FECHAINICIO, f_inicio),
					   fechafin = DECODE(f_fin, '', FECHAFIN, NULL, FECHAFIN, f_fin),
   					   idcentro = DECODE(id_centro, 0, IDCENTRO, NULL, IDCENTRO, id_centro)
					   WHERE idcurso = id;

	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
		
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: El curso no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
		
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar un curso: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaAsignatura (
v_nom IN varchar2,
cred IN number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaAsignatura';

BEGIN

 	params := v_nom || ',' || cred;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (v_nom IS NULL OR v_nom = '') OR
   	   (cred IS NULL OR cred = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO ASIGNATURAS (nombre, creditos)
	VALUES (v_nom, cred);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: La asignatura ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar la asignatura';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta una asignatura: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/



CREATE OR REPLACE PROCEDURE p_bajaAsignatura (
id IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaAsignatura';

BEGIN

 	params := id;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE ASIGNATURAS WHERE idasignatura = id;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
   END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: La asignatura no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja una asignatura: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modAsignatura (
id IN NUMBER,
RSP OUT Varchar2,
v_nom IN Varchar2 DEFAULT '',
cred IN number DEFAULT 0
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modAsignatura';

BEGIN

 	params := id || ',' || v_nom || ',' || cred;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE ASIGNATURAS SET nombre = DECODE(v_nom, '', NOMBRE, NULL, NOMBRE, v_nom),
		   		   	   creditos = DECODE(cred, 0, CREDITOS, NULL, CREDITOS, cred)
					   WHERE idasignatura = id;	 

					   
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar la asignatura';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN noencontrado THEN
	 	RSP := 'ERROR: La asignatura no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar una asignatura: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaProfesorCurso (
id_profesor IN number,
id_curso IN number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
noexiste EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaProfesorCurso';
PRAGMA EXCEPTION_INIT(noexiste, -2291);

BEGIN

 	params := id_profesor || ',' || id_curso;
    SELECT SYSDATE INTO f_ini FROM DUAL;

	IF (id_profesor IS NULL OR id_profesor = 0) OR
	   (id_curso IS NULL OR id_curso = 0) THEN
		 RAISE obligatorio;
	END IF;

	INSERT INTO PROFESORESCURSO (idprofesor, idcurso, idhorario)
	VALUES (id_profesor, id_curso, 0);

	RSP := 'OK';

	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

EXCEPTION
    WHEN noexiste THEN
		RSP := 'ERROR:La asignatura o el curso no existen';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: El profesor para ese curso ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);


	WHEN STORAGE_ERROR THEN

		RSP := 'ERROR: No se pudo insertar el profesor para ese curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

	WHEN obligatorio THEN

		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

	WHEN OTHERS THEN

		RSP := 'ERROR: Error al dar de alta un profesor en un curso: '  || sqlerrm ;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
END;
/

CREATE OR REPLACE PROCEDURE p_bajaProfesorCurso (
id_curso IN NUMBER,
id_profesor IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaProfesorCurso';

BEGIN

 	params := id_curso || ',' || id_profesor;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR 
	   (id_profesor IS NULL OR id_profesor = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE PROFESORESCURSO WHERE idcurso = id_curso and idprofesor = id_profesor;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: El profesor no existe en este curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un profesor en este curso: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_altaProfesorAsignatura (
id_profesor IN number,
id_asignatura IN number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
noexiste EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaProfesorAsignatura';
PRAGMA EXCEPTION_INIT(noexiste, -2291);

BEGIN

 	params := id_profesor || ',' || id_asignatura;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_profesor IS NULL OR id_profesor = 0) OR
	   (id_asignatura IS NULL OR id_asignatura = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO PROFESORESASIGNATURA (idprofesor, idasignatura)
	VALUES (id_profesor, id_asignatura);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
    WHEN noexiste THEN
		RSP := 'ERROR:La asignatura o el curso no existen';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El profesor para esta asignatura ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el profesor para esta asignatura';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un profesor en una asignatura: '  || sqlerrm ;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
END;
/

CREATE OR REPLACE PROCEDURE p_bajaProfesorAsignatura (
id_asignatura IN NUMBER,
id_profesor IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaProfesorAsignatura';

BEGIN

 	params := id_asignatura || ',' || id_profesor;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_asignatura IS NULL OR id_asignatura = 0) OR 
	   (id_profesor IS NULL OR id_profesor = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE PROFESORESASIGNATURA WHERE idasignatura = id_asignatura and idprofesor = id_profesor;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: El profesor no es docente en esta asignatura';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un profesor en esta asignatura: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaAsignaturaCurso (
id_curso IN number,
id_asignatura IN number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
noexiste EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaAsignaturaCurso';
PRAGMA EXCEPTION_INIT(noexiste, -2291);

BEGIN

 	params := id_curso || ',' || id_asignatura;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_asignatura IS NULL OR id_asignatura = 0) OR
	   (id_curso IS NULL OR id_curso = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO ASIGNATURASCURSO (idcurso, idasignatura)
	VALUES (id_curso, id_asignatura);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
    WHEN noexiste THEN
		RSP := 'ERROR:La asignatura o el curso no existen';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: La asignatura para ese curso ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar la asignatura para ese curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta una asignatura en un curso: '  || sqlerrm ;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
END;
/


CREATE OR REPLACE PROCEDURE p_bajaAsignaturaCurso (
id_curso IN NUMBER,
id_asignatura IN NUMBER,
RSP OUT Varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaAsignaturaCurso';

BEGIN

 	params := id_curso || ',' || id_asignatura;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_asignatura IS NULL OR id_asignatura = 0) OR 
	   (id_curso IS NULL OR id_curso = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE ASIGNATURASCURSO WHERE idasignatura = id_asignatura and idcurso = id_curso;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	
		RSP := 'ERROR: La asignatura no existe en este curso';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja una asignatura en este curso: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/





CREATE OR REPLACE PROCEDURE p_altaHorarioAsignatura (
id_curso IN number,
id_asignatura IN number,
dia_semana IN number,
hora_inicio IN varchar2,
hora_fin IN varchar2,
RSP OUT varchar2) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaHorarioAsignatura';

id_horario number(8) := 0;
aux number;
d_aux date;
d_aux2 date;



BEGIN

 	params := id_curso || ',' || id_asignatura || ',' || dia_semana || ',' || hora_inicio || ',' || hora_fin;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR
	   (id_asignatura IS NULL OR id_asignatura = 0) OR
	   (dia_semana IS NULL OR dia_semana = 0) OR
	   (hora_inicio IS NULL OR hora_inicio = '') OR
	   (hora_fin IS NULL OR hora_fin = '') THEN
		 RAISE obligatorio;
	END IF;
	
	BEGIN
		 SELECT TO_DATE(hora_inicio, 'HH24:MI'), TO_DATE(hora_fin, 'HH24:MI')
		 INTO d_aux, d_aux2
		 FROM DUAL;
		 
		 EXCEPTION WHEN OTHERS THEN 
		 		RSP := 'ERROR: Formato incorrecto de hora';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
	END;	
	
	BEGIN
		 SELECT IDASIGNATURA
 		 INTO aux
		 FROM ASIGNATURASCURSO
		 WHERE IDASIGNATURA = id_asignatura AND IDCURSO = id_curso;
		 
 		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra la asignatura para ese curso.
  				RSP := 'ERROR: La asignatura no se imparte en este curso';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
			 
	END;	
	
	BEGIN

		SELECT IDHORARIO 
		INTO id_horario
		FROM HORARIOS WHERE DIA = dia_semana AND HORAINICIO = hora_inicio AND HORAFIN = hora_fin; 

		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra el horario, es nuevo, se inserta.
			   	INSERT INTO HORARIOS(DIA, HORAINICIO, HORAFIN)
				   				VALUES (dia_semana, hora_inicio, hora_fin);
					
				SELECT IDHORARIO 
				INTO id_horario
		    	FROM HORARIOS WHERE DIA = dia_semana AND HORAINICIO = hora_inicio AND HORAFIN = hora_fin;
		WHEN OTHERS THEN 		
		DBMS_OUTPUT.PUT_LINE(sqlcode);

				RSP := 'ERROR: Error al dar de alta un horario: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;

	END;
	

    INSERT INTO HORARIOASIGNATURAS(IDCURSO, IDASIGNATURA, IDHORARIO)
     	          				   VALUES(id_curso, id_asignatura, id_horario);
								   
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El horario ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el horario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un horario: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_bajaHorarioAsignatura (
id_curso IN number,
id_asignatura IN number,
id_horario IN number,
RSP OUT varchar2) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaHorarioAsignatura';

aux number(8);



BEGIN

 	params := id_curso || ',' || id_asignatura || ',' || id_horario;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR
	   (id_asignatura IS NULL OR id_asignatura = 0) OR
	   (id_horario IS NULL OR id_horario = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	BEGIN
		 SELECT IDASIGNATURA
 		 INTO aux
		 FROM ASIGNATURASCURSO
		 WHERE IDASIGNATURA = id_asignatura AND IDCURSO = id_curso;
		 
 		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra la asignatura para ese curso.
  				RSP := 'ERROR: La asignatura no se imparte en este curso';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
			 
	END;	
	
	DELETE FROM HORARIOASIGNATURAS WHERE IDASIGNATURA = id_asignatura AND IDCURSO = id_curso AND IDHORARIO = id_horario;

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar el horario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un horario: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaHorarioPadresProf (
id_curso IN number,
id_profesor IN number,
dia_semana IN number,
hora_inicio IN varchar2,
hora_fin IN varchar2,
RSP OUT varchar2) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaHorarioPadresProf';

id_horario number(8) := 0;
aux number(8);
d_aux date;
d_aux2 date;



BEGIN

 	params := id_curso || ',' || id_profesor || ',' || dia_semana || ',' || hora_inicio || ',' || hora_fin;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR
	   (id_profesor IS NULL OR id_profesor = 0) OR
	   (dia_semana IS NULL OR dia_semana = 0) OR
	   (hora_inicio IS NULL OR hora_inicio = '') OR
	   (hora_fin IS NULL OR hora_fin = '') THEN
		 RAISE obligatorio;
	END IF;
	
	BEGIN
		 SELECT TO_DATE(hora_inicio, 'HH24:MI'), TO_DATE(hora_fin, 'HH24:MI')
		 INTO d_aux, d_aux2
		 FROM DUAL;
		 
		 EXCEPTION WHEN OTHERS THEN 
		 		RSP := 'ERROR: Formato incorrecto de hora';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
	END;	

	
	BEGIN
		 SELECT DISTINCT IDPROFESOR
 		 INTO aux
		 FROM PROFESORESCURSO
		 WHERE IDPROFESOR = id_profesor AND IDCURSO = id_curso;
		 
 		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra el profesor para ese curso.
  				RSP := 'ERROR: El profesor no es el responsable de este curso';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
			 
	END;	
	
	BEGIN

		SELECT IDHORARIO 
		INTO id_horario
		FROM HORARIOS WHERE DIA = dia_semana AND HORAINICIO = hora_inicio AND HORAFIN = hora_fin; 

		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra el horario, es que es nuevo, se inserta.
			   	INSERT INTO HORARIOS(DIA, HORAINICIO, HORAFIN)
				   				VALUES (dia_semana, hora_inicio, hora_fin);
					
				SELECT IDHORARIO 
				INTO id_horario
		    	FROM HORARIOS WHERE DIA = dia_semana AND HORAINICIO = hora_inicio AND HORAFIN = hora_fin;
		WHEN OTHERS THEN 		

				RSP := 'ERROR: Error al dar de alta un horario: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;

	END;
	

	INSERT INTO PROFESORESCURSO (IDPROFESOR, IDCURSO, IDHORARIO, TIPOHORARIO) 
		   		VALUES(id_profesor, id_curso, id_horario, 'P');

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El horario ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el horario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un horario: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_bajaHorarioPadresProf (
id_curso IN number,
id_profesor IN number,
id_horario IN number,
RSP OUT varchar2) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaHorarioPadresProf';

aux number(8);



BEGIN

 	params := id_curso || ',' || id_profesor || ',' || id_horario;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR
	   (id_profesor IS NULL OR id_profesor = 0) OR
	   (id_horario IS NULL OR id_horario = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	BEGIN
		 SELECT DISTINCT IDPROFESOR
 		 INTO aux
		 FROM PROFESORESCURSO
		 WHERE IDPROFESOR = id_profesor AND IDCURSO = id_curso;
		 
 		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra el profesor para ese curso.
  				RSP := 'ERROR: El profesor no es el responsable de este curso';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
			 
	END;	
	
	DELETE FROM PROFESORESCURSO WHERE IDPROFESOR = id_profesor AND IDCURSO = id_curso AND IDHORARIO = id_horario AND TIPOHORARIO = 'P';

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar el horario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un horario: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_altaHorarioAlumnoProf (
id_curso IN number,
id_profesor IN number,
dia_semana IN number,
hora_inicio IN varchar2,
hora_fin IN varchar2,
RSP OUT varchar2) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaHorarioAlumnoProf';

id_horario number(8) := 0;
aux number(8);
d_aux date;
d_aux2 date;


BEGIN

 	params := id_curso || ',' || id_profesor || ',' || dia_semana || ',' || hora_inicio || ',' || hora_fin;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR
	   (id_profesor IS NULL OR id_profesor = 0) OR
	   (dia_semana IS NULL OR dia_semana = 0) OR
	   (hora_inicio IS NULL OR hora_inicio = '') OR
	   (hora_fin IS NULL OR hora_fin = '') THEN
		 RAISE obligatorio;
	END IF;
	
	BEGIN
		 SELECT TO_DATE(hora_inicio, 'HH24:MI'), TO_DATE(hora_fin, 'HH24:MI')
		 INTO d_aux, d_aux2
		 FROM DUAL;
		 
		 EXCEPTION WHEN OTHERS THEN 
		 		RSP := 'ERROR: Formato incorrecto de hora';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
	END;	
	
	BEGIN
		 SELECT DISTINCT IDPROFESOR
 		 INTO aux
		 FROM PROFESORESCURSO
		 WHERE IDPROFESOR = id_profesor AND IDCURSO = id_curso;
		 
 		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra el profesor para ese curso.
  				RSP := 'ERROR: El profesor no es el responsable de este curso';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
			 
	END;	
	
	BEGIN

		SELECT IDHORARIO 
		INTO id_horario
		FROM HORARIOS WHERE DIA = dia_semana AND HORAINICIO = hora_inicio AND HORAFIN = hora_fin; 

		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra el horario, es que es nuevo, se inserta.
			   	INSERT INTO HORARIOS(DIA, HORAINICIO, HORAFIN)
				   				VALUES (dia_semana, hora_inicio, hora_fin);
					
				SELECT IDHORARIO 
				INTO id_horario
		    	FROM HORARIOS WHERE DIA = dia_semana AND HORAINICIO = hora_inicio AND HORAFIN = hora_fin;
		WHEN OTHERS THEN 		

				RSP := 'ERROR: Error al dar de alta un horario: ' || sqlerrm;
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;

	END;
	

	INSERT INTO PROFESORESCURSO (IDPROFESOR, IDCURSO, IDHORARIO, TIPOHORARIO) 
		   		VALUES(id_profesor, id_curso, id_horario, 'A');

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El horario ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el horario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un horario: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_bajaHorarioAlumnoProf (
id_curso IN number,
id_profesor IN number,
id_horario IN number,
RSP OUT varchar2) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaHorarioAlumnoProf';

aux number(8);



BEGIN

 	params := id_curso || ',' || id_profesor || ',' || id_horario;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_curso IS NULL OR id_curso = 0) OR
	   (id_profesor IS NULL OR id_profesor = 0) OR
	   (id_horario IS NULL OR id_horario = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	BEGIN
		 SELECT DISTINCT IDPROFESOR
 		 INTO aux
		 FROM PROFESORESCURSO
		 WHERE IDPROFESOR = id_profesor AND IDCURSO = id_curso;
		 
 		EXCEPTION WHEN NO_DATA_FOUND THEN --Si no se encuentra el profesor para ese curso.
  				RSP := 'ERROR: El profesor no es el responsable de este curso';
				ROLLBACK;
				p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
				RETURN;
			 
	END;	
	
	DELETE FROM PROFESORESCURSO WHERE IDPROFESOR = id_profesor AND IDCURSO = id_curso AND IDHORARIO = id_horario AND TIPOHORARIO = 'A';

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar el horario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un horario: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaFestivo (
fecha_f IN date,
descripcion IN varchar2,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaFestivo';

BEGIN

 	params := fecha_f || ',' || descripcion;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (fecha_f IS NULL OR fecha_f = '') THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE FECHASCURSO SET festivo = 'S', desc_festivo = descripcion
		   			   WHERE fecha = fecha_f;

	IF SQL%ROWCOUNT = 0 THEN
	   RAISE noencontrado;
	END IF;
					   
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el festivo';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN noencontrado THEN
	 	RSP := 'ERROR: La fecha no pertenece al calendario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un festivo: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_bajaFestivo (
fecha_f IN date,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaFestivo';

BEGIN

 	params := fecha_f;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (fecha_f IS NULL OR fecha_f = '') THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE FECHASCURSO SET festivo = 'N'
		   			   WHERE fecha = fecha_f;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar el festivo';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El festivo no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un festivo: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modFestivo (
fecha_f IN date,
descripcion IN varchar2,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
noencontrado EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modFestivo';

BEGIN

 	params := fecha_f || ',' || descripcion;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (fecha_f IS NULL OR fecha_f = '') THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE FECHASCURSO SET festivo = 'S', desc_festivo = descripcion
		   			   WHERE fecha = fecha_f;

	IF SQL%ROWCOUNT = 0 THEN
	   RAISE noencontrado;
	END IF;
					   
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el festivo';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN noencontrado THEN
	 	RSP := 'ERROR: La fecha no pertenece al calendario';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar un festivo: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaTipoAmonestacion (
id_profesor IN number,
descrip IN varchar2,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaTipoAmonestacion';

BEGIN

 	params := id_profesor || ',' || descrip;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_profesor IS NULL OR id_profesor = 0) OR
	   (descrip IS NULL OR descrip = '')THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO TIPOAMONESTACIONES (idprofesor, descripcion)
	VALUES (id_profesor, descrip);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El tipo de amonestacin ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el tipo de amonestacin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un tipo de amonestacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_bajaTipoAmonestacion (
id_tipo IN number,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaTipoAmonestacion';

BEGIN

 	params := id_tipo;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_tipo IS NULL OR id_tipo = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE TIPOAMONESTACIONES WHERE idtipoamo = id_tipo;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	 WHEN noencontrado THEN
	
		RSP := 'ERROR: El tipo de amonestacin no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar el tipo de amonestacin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un tipo de amonestacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modTipoAmonestacion (
id_tipo IN number,
id_profesor IN number,
descrip IN varchar2,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modTipoAmonestacion';

BEGIN

 	params := id_tipo;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_tipo IS NULL OR id_tipo = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE TIPOAMONESTACIONES SET idprofesor = DECODE(id_profesor, 0, idprofesor, NULL, idprofesor, id_profesor),
		   					      descripcion = DECODE(descrip, '', descripcion, NULL, descripcion, descrip)
						      WHERE idtipoamo = id_tipo;
							  
	IF (SQL%ROWCOUNT) = 0 THEN
	   RAISE noencontrado;
    END IF; 
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El tipo de amonestacin no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el tipo de amonestacin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar un tipo de amonestacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaTipoSancion (
id_profesor IN number,
descrip IN varchar2,
id_tipoamo IN number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaTipoSancion';

BEGIN

 	params := id_profesor || ',' || descrip || ',' || id_tipoamo;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_profesor IS NULL OR id_profesor = 0) OR
	   (descrip IS NULL OR descrip = '') OR 
	   (id_tipoamo IS NULL OR id_tipoamo = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO TIPOSANCIONES (idprofesor, descripcion, idtipoamorelacionada)
	VALUES (id_profesor, descrip, id_tipoamo);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: El tipo de sancin ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar el tipo de sancin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta un tipo de sancin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_bajaTipoSancion (
id_tipo IN number,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaTipoSancion';

BEGIN

 	params := id_tipo;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_tipo IS NULL OR id_tipo = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE TIPOSANCIONES WHERE idtiposan = id_tipo;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
    WHEN noencontrado THEN
	
		RSP := 'ERROR: El tipo de sancin no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar el tipo de sancin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja un tipo de sancin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modTipoSancion (
id_tipo IN number,
id_profesor IN number,
descrip IN varchar2,
id_tipoamo number,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modTipoSancion';

BEGIN

 	params := id_tipo;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_tipo IS NULL OR id_tipo = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE TIPOSANCIONES SET idprofesor = DECODE(id_profesor, 0, idprofesor, NULL, idprofesor, id_profesor),
		   					      descripcion = DECODE(descrip, '', descripcion, NULL, descripcion, descrip),
								  idtipoamorelacionada = DECODE(id_tipoamo, 0, idtipoamorelacionada, NULL, idtipoamorelacionada, id_tipoamo)
						      WHERE idtiposan = id_tipo;
							  
	IF (SQL%ROWCOUNT) = 0 THEN
	   RAISE noencontrado;
    END IF; 
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	 	RSP := 'ERROR: El tipo de sancin no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar el tipo de sancin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar un tipo de sancin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaAmonestacion (
id_alumno IN number,
id_curso IN number,
id_asignatura IN number,
id_profesor IN number,
id_tipo IN number,
RSP OUT varchar2,
fecha_amo IN date DEFAULT SYSDATE,
gravedad_amo IN varchar2 DEFAULT '',
comunicado_amo IN varchar2 DEFAULT 'N'
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaAmonestacion';

BEGIN

 	params := id_alumno || ',' || id_curso || ',' || id_asignatura || ',' || id_profesor || ',' || id_tipo || ',' || fecha_amo || ',' || gravedad_amo || ',' || comunicado_amo;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
IF (id_alumno IS NULL OR id_alumno = 0) OR
       (id_curso IS NULL OR id_curso = 0) OR
	   (id_asignatura IS NULL OR id_asignatura = 0) OR
	   (id_profesor IS NULL OR id_profesor = 0) OR
	   (id_tipo IS NULL OR id_tipo = 0) OR
	   (fecha_amo IS NULL OR fecha_amo = '') THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO AMONESTACIONES (idalumno, idcurso, idasignatura, idprofesor, idtipo, fecha, gravedad, comunicado)
	VALUES (id_alumno, id_curso, id_asignatura, id_profesor, id_tipo, fecha_amo, gravedad_amo, comunicado_amo);
	
	p_rellenaEstadisticasAmo;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: La amonestacin ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar la amonestacin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta una amonestacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaSancion (
id_alumno IN number,
id_curso IN number,
id_profesor IN number,
id_tipo IN number,
RSP OUT varchar2,
fecha_san IN date DEFAULT SYSDATE
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaSancion';

BEGIN

 	params := id_alumno || ',' || id_curso || ',' || id_profesor || ',' || id_tipo || ',' || fecha_san;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id_alumno IS NULL OR id_alumno = 0) OR
       (id_curso IS NULL OR id_curso = 0) OR
	   (id_profesor IS NULL OR id_profesor = 0) OR
	   (id_tipo IS NULL OR id_tipo = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO SANCIONES (idalumno, idcurso, idprofesor, idtiposan, fecha, idreglaaplicada)
	VALUES (id_alumno, id_curso, id_profesor, id_tipo, fecha_san, NULL);
	
	p_rellenaEstadisticasSan;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: La sancin ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar la sancin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta una sancin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_bajaAmonestacion (
id IN number,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaAmonestacion';

BEGIN

 	params := id;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE AMONESTACIONES WHERE idamonestacion = id;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	p_rellenaEstadisticasAmo;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
    WHEN noencontrado THEN
	
		RSP := 'ERROR: La amonestacin no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar la amonestacin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja una amonestacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_bajaSancion (
id IN number,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaSancion';

BEGIN

 	params := id;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE SANCIONES WHERE idsancion = id;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	p_rellenaEstadisticasSan;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
    WHEN noencontrado THEN
	
		RSP := 'ERROR: La sancin no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar la sancin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja una sancin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modAmonestacion (
id IN number,
RSP OUT varchar2,
id_alumno IN number DEFAULT 0,
id_curso IN number DEFAULT 0,
id_asignatura IN number DEFAULT 0,
id_profesor IN number DEFAULT 0,
id_tipo IN number DEFAULT 0,
fecha_amo IN date DEFAULT '',
gravedad_amo IN varchar2 DEFAULT '',
comunicado_amo IN varchar2 DEFAULT ''
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modAmonestacion';

BEGIN

 	params := id || ',' || id_alumno || ',' || id_curso || ',' || id_asignatura || ',' || id_profesor || ',' || id_tipo || ',' || fecha_amo || ',' || gravedad_amo || ',' || comunicado_amo;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE AMONESTACIONES SET idalumno = DECODE(id_alumno, 0, idalumno, NULL, idalumno, id_alumno),
		   				  	  idcurso = DECODE(id_curso, 0, idcurso, NULL, idcurso, id_curso),
							  idasignatura = DECODE(id_asignatura, 0, idasignatura, NULL, idasignatura, id_asignatura),
							  idprofesor = DECODE(id_profesor, 0, idprofesor, NULL, idprofesor, id_profesor),
							  idtipo = DECODE(id_tipo, 0, idtipo, NULL, idtipo, id_tipo),
							  fecha = DECODE(fecha_amo, '', fecha, NULL, fecha, fecha_amo),
							  gravedad = DECODE(gravedad_amo, '', gravedad, NULL, gravedad, gravedad_amo),
							  comunicado = DECODE(comunicado_amo, '', comunicado, NULL, comunicado, comunicado_amo)
							  WHERE idamonestacion = id;
	
	IF SQL%ROWCOUNT = 0 THEN
	   RAISE noencontrado;
	END IF;
	
	p_rellenaEstadisticasAmo;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	 	RSP := 'ERROR: La amonestacion no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
			
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar la amonestacin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar una amonestacin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modSancion (
id IN number,
RSP OUT varchar2,
id_alumno IN number DEFAULT 0,
id_curso IN number DEFAULT 0,
id_profesor IN number DEFAULT 0,
id_tipo IN number DEFAULT 0,
id_regla IN number DEFAULT 0,
fecha_san IN date DEFAULT '',
resolucion_san IN varchar2 DEFAULT ''
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modSancion';

BEGIN

 	params := id || ',' || id_alumno || ',' || id_curso || ',' || id_profesor || ',' || id_tipo || ',' || id_regla || ',' || fecha_san  || ',' || resolucion_san;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE SANCIONES SET idalumno = DECODE(id_alumno, 0, idalumno, NULL, idalumno, id_alumno),
		   				  	  idcurso = DECODE(id_curso, 0, idcurso, NULL, idcurso, id_curso),
							  idprofesor = DECODE(id_profesor, 0, idprofesor, NULL, idprofesor, id_profesor),
							  idtiposan = DECODE(id_tipo, 0, idtiposan, NULL, idtiposan, id_tipo),
							  idreglaaplicada = DECODE(id_regla, 0, idreglaaplicada, NULL, idreglaaplicada, id_regla),
							  fecha = DECODE(fecha_san, '', fecha, NULL, fecha, fecha_san),
							  resolucion = DECODE(resolucion_san, '', resolucion, NULL, resolucion, resolucion_san)
							  WHERE idsancion = id;
	
	IF SQL%ROWCOUNT = 0 THEN
	   RAISE noencontrado;
	END IF;
	
	p_rellenaEstadisticasSan;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	 	RSP := 'ERROR: La sancin no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
			
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar la sancin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar una sancin: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_altaRegla (
tipo_amo IN number,
tipo_san IN number,
oper IN varchar2,
valor IN number,
RSP OUT varchar2
) IS

obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_altaRegla';

BEGIN

 	params := tipo_amo || ',' || tipo_san || ',' || oper || ',' || valor;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (tipo_amo IS NULL OR tipo_amo = 0) OR
       (tipo_san IS NULL OR tipo_san = 0) OR
	   (oper IS NULL OR oper = '') OR
	   (valor IS NULL OR valor = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	INSERT INTO REGLASSANCIONES (idtipoamo, idtiposan, operando, valornumerico)
	VALUES (tipo_amo, tipo_san, oper, valor);
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
	
		RSP := 'ERROR: La regla ya existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo insertar la regla';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de alta una regla: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_bajaRegla (
id IN number,
RSP OUT varchar2
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_bajaRegla';

BEGIN

 	params := id;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	DELETE REGLASSANCIONES WHERE idregla = id;
	
	IF SQL%NOTFOUND THEN
	   RAISE noencontrado;
    END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
    WHEN noencontrado THEN
	
		RSP := 'ERROR: La regla no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo eliminar la regla';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al dar de baja una regla' || sqlcode || ':' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/


CREATE OR REPLACE PROCEDURE p_modRegla (
id IN number,
RSP OUT varchar2,
tipo_amo IN number DEFAULT 0,
tipo_san IN number DEFAULT 0,
oper IN varchar2 DEFAULT '',
valor IN number DEFAULT 0
) IS

noencontrado EXCEPTION;
obligatorio EXCEPTION;
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_modRegla';

BEGIN

	params := id || ',' || tipo_amo || ',' || tipo_san || ',' || oper || ',' || valor;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	IF (id IS NULL OR id = 0) THEN
		 RAISE obligatorio;
	END IF;
	
	UPDATE REGLASSANCIONES SET idtipoamo = DECODE(tipo_amo, 0, idtipoamo, NULL, idtipoamo, tipo_amo),
		   				  	  idtiposan = DECODE(tipo_san, 0, idtiposan, NULL, idtiposan, tipo_san),
							  operando = DECODE(oper, '', operando, NULL, operando, oper),
							  valornumerico = DECODE(valor, 0, valornumerico, NULL, valornumerico, valor)
							  WHERE idregla = id;
	
	IF SQL%ROWCOUNT = 0 THEN
	   RAISE noencontrado;
	END IF;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN noencontrado THEN
	 	RSP := 'ERROR: La regla no existe';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
			
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No se pudo modificar la regla';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN obligatorio THEN
	
		RSP := 'ERROR: Faltan campos obligatorios';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al modificar una regla: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END;
/

CREATE OR REPLACE PROCEDURE p_rellenaEstadisticasAmo
 IS
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_rellenaEstadisticasAmo';
d_anyo varchar2(4);
id_curso number(8);
RSP varchar2(500);

CURSOR CURSOSAMO IS SELECT DISTINCT IDCURSO FROM CURSOS;

BEGIN

 	params := '';
    SELECT SYSDATE INTO f_ini FROM DUAL;


	-- Borramos todas las tablas de etadsticas para refrescar los datos	
	DELETE EST_AMOXALUMNO;
	DELETE EST_MEDIAAMOPROFESORESXANYO;
	DELETE EST_PROFESORMASAMO;
	DELETE EST_ALUMNOSSINAMO;


	-- Insertamos amonestaciones por alumno
	 INSERT INTO EST_AMOXALUMNO (SELECT COUNT(1), IDALUMNO FROM AMONESTACIONES GROUP BY IDALUMNO);
	 
	 -- Insertamos la media de amonestaciones por profesor y ao
	 INSERT INTO EST_MEDIAAMOPROFESORESXANYO (SELECT IDPROFESOR, AVG(NUMAMO) 
	 			 			 					   FROM (SELECT COUNT(1) NUMAMO, EXTRACT(YEAR FROM FECHA), IDPROFESOR 
												   						   		 FROM AMONESTACIONES GROUP BY EXTRACT(YEAR FROM FECHA), IDPROFESOR) GROUP BY IDPROFESOR);
	 
 	 OPEN CURSOSAMO;
	 
	 FETCH CURSOSAMO INTO id_curso;
	 
	 --Recorremos todos los cursos para ir rellenando su profesor ms amonestador
	 WHILE CURSOSAMO%FOUND LOOP
	 
	 	 	 -- Insertamos el profesor ms sancionador por curso
	 	   INSERT INTO EST_PROFESORMASAMO (SELECT NUMAMO, AMO.IDPROFESOR, NOMBRE, id_curso FROM (SELECT COUNT(1) NUMAMO, IDPROFESOR FROM AMONESTACIONES
								  		  WHERE IDCURSO = id_curso GROUP BY IDPROFESOR) AMO, PROFESORES PR  
										  	   WHERE NUMAMO =(SELECT MAX(NUMAMO) FROM (SELECT COUNT(1) NUMAMO FROM AMONESTACIONES WHERE IDCURSO = id_curso GROUP BY IDPROFESOR))
   											   	AND AMO.IDPROFESOR = PR.IDPROFESOR);
	 
	 	 	 FETCH CURSOSAMO INTO id_curso;
	 END LOOP;
	 
	 CLOSE CURSOSAMO;
	 
 	 -- Insertamos el nmero de alumnos sin amonestaciones
	 INSERT INTO EST_ALUMNOSSINAMO (SELECT COUNT(1) FROM ALUMNOS AL WHERE not exists (SELECT IDALUMNO FROM AMONESTACIONES AM WHERE AM.IDALUMNO = AL.IDALUMNO));
	 
	 RSP := 'OK';
     p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	 
EXCEPTION
	WHEN OTHERS THEN
		RSP := 'ERROR: Error al rellenar estadsticas: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
END;
/

CREATE OR REPLACE PROCEDURE p_rellenaEstadisticasSan
 IS
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_rellenaEstadisticasSan';
d_anyo varchar2(4);
id_curso number(8);
RSP varchar2(500);

CURSOR ANYOSSAN IS SELECT DISTINCT EXTRACT(YEAR FROM FECHA) FROM SANCIONES;

BEGIN

 	params := '';
    SELECT SYSDATE INTO f_ini FROM DUAL;


	-- Borramos todas las tablas de etadsticas para refrescar los datos	
	DELETE EST_SANXALUMNOXANYO;
	DELETE EST_SANXANYOXCURSO;
	DELETE EST_ALUMNOMASSAN;
	DELETE EST_MEDIASANALUMNOSXCURSO;
	
	 
	 -- Insertamos sanciones por alumno y ao
	 INSERT INTO EST_SANXALUMNOXANYO (SELECT COUNT(1), IDALUMNO, EXTRACT(YEAR FROM FECHA) FROM SANCIONES GROUP BY IDALUMNO, EXTRACT(YEAR FROM FECHA));

	 -- Insertamos el nmero de sanciones por curso y ao
	 INSERT INTO EST_SANXANYOXCURSO (SELECT COUNT(1), IDCURSO, EXTRACT(YEAR FROM FECHA) FROM SANCIONES GROUP BY IDCURSO, EXTRACT(YEAR FROM FECHA));
	 
	 OPEN ANYOSSAN;
	 
	 FETCH ANYOSSAN INTO d_anyo;
	 
	 --Recorremos todos los aos para ir rellenando su alumno ms sancionado
	 WHILE ANYOSSAN%FOUND LOOP

	 	 -- Insertamos el alumno ms sancionado por ao
	 	    INSERT INTO EST_ALUMNOMASSAN (SELECT NUMSAN, SAN.IDALUMNO, NOMBRE, d_anyo  FROM  (SELECT COUNT(1) NUMSAN, IDALUMNO FROM SANCIONES
								  		  WHERE EXTRACT(YEAR FROM FECHA) = d_anyo GROUP BY IDALUMNO) SAN, ALUMNOS AL  
										  	   WHERE NUMSAN =  (SELECT MAX(NUMSAN) FROM (SELECT COUNT(1) NUMSAN FROM SANCIONES WHERE EXTRACT(YEAR FROM FECHA) = d_anyo GROUP BY IDALUMNO)) 
											   	AND SAN.IDALUMNO = AL.IDALUMNO);  
	 	 FETCH ANYOSSAN INTO d_anyo;
	 END LOOP;
	 
	 CLOSE ANYOSSAN;
	 
	 
 
 	 -- Insertamos la media de sanciones de alumnos por curso
	 INSERT INTO EST_MEDIASANALUMNOSXCURSO (SELECT IDALUMNO, AVG(NUMSAN) 
	 			 					   	   		   FROM (SELECT COUNT(1) NUMSAN, IDCURSO, IDALUMNO 
												   		 FROM SANCIONES GROUP BY IDCURSO, IDALUMNO) GROUP BY IDALUMNO);
	 

	 RSP := 'OK';
     p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	 
EXCEPTION
	WHEN OTHERS THEN
		RSP := 'ERROR: Error al rellenar estadsticas: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
END;
/

CREATE OR REPLACE PROCEDURE p_rellenaEstadisticas IS

f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_rellenaEstadisticas';
d_anyo varchar2(4);
id_curso number(8);
RSP varchar2(500);

CURSOR ANYOSSAN IS SELECT DISTINCT EXTRACT(YEAR FROM FECHA) FROM SANCIONES;
CURSOR CURSOSAMO IS SELECT DISTINCT IDCURSO FROM CURSOS;

BEGIN

 	params := '';
    SELECT SYSDATE INTO f_ini FROM DUAL;


	-- Borramos todas las tablas de etadsticas para refrescar los datos	
	DELETE EST_AMOXALUMNO;
	DELETE EST_SANXALUMNOXANYO;
	DELETE EST_MEDIAAMOPROFESORESXANYO;
	DELETE EST_SANXANYOXCURSO;
	DELETE EST_ALUMNOMASSAN;
	DELETE EST_PROFESORMASAMO;
	DELETE EST_MEDIASANALUMNOSXCURSO;
	DELETE EST_ALUMNOSSINAMO;
	
	



	-- Insertamos amonestaciones por alumno
	 INSERT INTO EST_AMOXALUMNO (SELECT COUNT(1), IDALUMNO FROM AMONESTACIONES GROUP BY IDALUMNO);
	 
	 -- Insertamos sanciones por alumno y ao
	 INSERT INTO EST_SANXALUMNOXANYO (SELECT COUNT(1), IDALUMNO, EXTRACT(YEAR FROM FECHA) FROM SANCIONES GROUP BY IDALUMNO, EXTRACT(YEAR FROM FECHA));
	 
	 -- Insertamos la media de amonestaciones por profesor y ao
	 INSERT INTO EST_MEDIAAMOPROFESORESXANYO (SELECT IDPROFESOR, AVG(NUMAMO) 
	 			 			 					   FROM (SELECT COUNT(1) NUMAMO, EXTRACT(YEAR FROM FECHA), IDPROFESOR 
												   						   		 FROM AMONESTACIONES GROUP BY EXTRACT(YEAR FROM FECHA), IDPROFESOR) GROUP BY IDPROFESOR);
	 -- Insertamos el nmero de sanciones por curso y ao
	 INSERT INTO EST_SANXANYOXCURSO (SELECT COUNT(1), IDCURSO, EXTRACT(YEAR FROM FECHA) FROM SANCIONES GROUP BY IDCURSO, EXTRACT(YEAR FROM FECHA));
	 
	 OPEN ANYOSSAN;
	 
	 FETCH ANYOSSAN INTO d_anyo;
	 
	 --Recorremos todos los aos para ir rellenando su alumno ms sancionado
	 WHILE ANYOSSAN%FOUND LOOP

	 	 -- Insertamos el alumno ms sancionado por ao
	 	    INSERT INTO EST_ALUMNOMASSAN (SELECT NUMSAN, SAN.IDALUMNO, NOMBRE, d_anyo  FROM  (SELECT COUNT(1) NUMSAN, IDALUMNO FROM SANCIONES
								  		  WHERE EXTRACT(YEAR FROM FECHA) = d_anyo GROUP BY IDALUMNO) SAN, ALUMNOS AL  
										  	   WHERE NUMSAN =  (SELECT MAX(NUMSAN) FROM (SELECT COUNT(1) NUMSAN FROM SANCIONES WHERE EXTRACT(YEAR FROM FECHA) = d_anyo GROUP BY IDALUMNO)) 
											   	AND SAN.IDALUMNO = AL.IDALUMNO);  
	 	 FETCH ANYOSSAN INTO d_anyo;
	 END LOOP;
	 
	 CLOSE ANYOSSAN;
	 
	 
 	 OPEN CURSOSAMO;
	 
	 FETCH CURSOSAMO INTO id_curso;
	 
	 --Recorremos todos los cursos para ir rellenando su profesor ms amonestador
	 WHILE CURSOSAMO%FOUND LOOP
	 
	 	 	 -- Insertamos el profesor ms sancionador por curso
	 	   INSERT INTO EST_PROFESORMASAMO (SELECT NUMAMO, AMO.IDPROFESOR, NOMBRE, id_curso FROM (SELECT COUNT(1) NUMAMO, IDPROFESOR FROM AMONESTACIONES
								  		  WHERE IDCURSO = id_curso GROUP BY IDPROFESOR) AMO, PROFESORES PR  
										  	   WHERE NUMAMO =(SELECT MAX(NUMAMO) FROM (SELECT COUNT(1) NUMAMO FROM AMONESTACIONES WHERE IDCURSO = id_curso GROUP BY IDPROFESOR))
   											   	AND AMO.IDPROFESOR = PR.IDPROFESOR);
	 
	 	 	 FETCH CURSOSAMO INTO id_curso;
	 END LOOP;
	 
	 CLOSE CURSOSAMO;
	 
 	 -- Insertamos la media de sanciones de alumnos por curso
	 INSERT INTO EST_MEDIASANALUMNOSXCURSO (SELECT IDALUMNO, AVG(NUMSAN) 
	 			 					   	   		   FROM (SELECT COUNT(1) NUMSAN, IDCURSO, IDALUMNO 
												   		 FROM SANCIONES GROUP BY IDCURSO, IDALUMNO) GROUP BY IDALUMNO);
	 
 	 -- Insertamos el nmero de alumnos sin amonestaciones
	 INSERT INTO EST_ALUMNOSSINAMO (SELECT COUNT(1) FROM ALUMNOS AL WHERE not exists (SELECT IDALUMNO FROM AMONESTACIONES AM WHERE AM.IDALUMNO = AL.IDALUMNO));
	 
	 RSP := 'OK';
     p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	 
EXCEPTION
	WHEN OTHERS THEN
		RSP := 'ERROR: Error al rellenar estadsticas: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
END;
/

DECLARE

job_num binary_integer;

BEGIN
DBMS_JOB.SUBMIT(job_num,'BEGIN p_rellenaEstadisticas; END;',
 TO_DATE(TO_CHAR(SYSDATE, 'DD/MM/YYYY') || ' 02:00:00','DD/MM/YYYY HH24:MI:SS'),
'SYSDATE + 1');

--dbms_output.put_line('NUMERO ASIGNADO: ' || to_char(job_num));
END;
/


CREATE OR REPLACE PROCEDURE p_comprobarReglas (
id_tipo IN number,
id_alumno IN number,
id_curso IN number,
RSP OUT varchar2
)
IS
	 id_regla number(4);
	 tipo_san number(4);
	 oper varchar2(2);
	 valor number(4);
	 num_amo number(3);
	 regla_cumplida boolean;
	 
	 f_ini DATE;
	 params VARCHAR2(500);
	 nomproc CONSTANT VARCHAR2(25) := 'p_comprobarReglas';
	 
	 CURSOR c_reglas IS SELECT IDREGLA, IDTIPOSAN, OPERANDO, VALORNUMERICO 
	 FROM REGLASSANCIONES
	 WHERE IDTIPOAMO = id_tipo;
BEGIN
	 
	params := id_tipo || ',' || id_alumno || ',' || id_curso;
    SELECT SYSDATE INTO f_ini FROM DUAL;
	 
	 -- Seleccionamos el nmero de amonestaciones de ese tipo que tiene ese alumno.  
	 -- Para volver a contar desde cero cuando se le aplique una sancin solo contamos las amonestaciones 
	 -- desde la fecha de su ltima sancin AUTOMTICA (no se tienen en cuenta las sanciones introducidas manualmente por un profesor) de ese tipo 
	SELECT COUNT(1) + 1
	INTO num_amo
	FROM AMONESTACIONES
	WHERE IDALUMNO = id_alumno
	AND IDTIPO = id_tipo
	AND FECHA > (SELECT NVL(MAX(FECHA), '01/01/1970') FROM SANCIONES S, TIPOSANCIONES T WHERE S.IDTIPOSAN = T.IDTIPOSAN 
			  						  				  	   			 				  	AND S.IDALUMNO = id_alumno
																						AND T.IDTIPOAMORELACIONADA = id_tipo
																						AND S.IDREGLAAPLICADA IS NOT NULL);
	 
	OPEN c_reglas;
	 
	-- Usamos un cursor porque puede haber ms de una regla para el mismo tipo de amonestacin
	FETCH c_reglas INTO id_regla, tipo_san, oper, valor;
	
	WHILE c_reglas%FOUND LOOP
	   	   CASE oper 
		   WHEN '<'	THEN
		   		IF num_amo < valor THEN
   		   		   regla_cumplida := true;
				END IF; 
   		   WHEN '>'	THEN
		   		IF num_amo > valor THEN
   		   		   regla_cumplida := true;
				END IF; 
   		   WHEN '='	THEN
		   		IF num_amo = valor THEN
   		   		   regla_cumplida := true;
				END IF; 
   		   WHEN '<=' THEN
		   		IF num_amo <= valor THEN
   		   		   regla_cumplida := true;
				END IF;
		   WHEN '=<' THEN
		        IF num_amo <= valor THEN
   		   		   regla_cumplida := true;
         	    END IF;  
   		   WHEN '>=' THEN
		   		IF num_amo >= valor THEN
   		   		   regla_cumplida := true;
				END IF;
   		   WHEN '=>' THEN
		   		IF num_amo >= valor THEN
   		   		   regla_cumplida := true;
				END IF;  
		   END CASE;

		   IF regla_cumplida THEN		   
		   	  INSERT INTO SANCIONES (idalumno, idcurso, idtiposan, fecha, idreglaaplicada)
		   	  VALUES (id_alumno, id_curso, tipo_san, SYSDATE, id_regla);	
		   
		      regla_cumplida := false;
		   END IF;
		   
		   FETCH c_reglas INTO id_regla, tipo_san, oper, valor;
 	   
	END LOOP;
	
	CLOSE c_reglas;
	
	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN

		RSP := 'ERROR: No se pudo insertar la sancin';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	

	
	WHEN STORAGE_ERROR THEN
	
		RSP := 'ERROR: No al comprobar las reglas de sanciones';
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al comprobar las reglas de sanciones: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
END;
/


CREATE OR REPLACE PACKAGE LISTADOS AS 
  
TYPE AMO_TABLA IS TABLE OF LISTADOAMONESTACIONES%ROWTYPE INDEX BY BINARY_INTEGER ;
TYPE ALUMXCUR_TABLA IS TABLE OF ALUMNOSXCURSO%ROWTYPE INDEX BY BINARY_INTEGER ;
TYPE AMOYSAN_TABLA IS TABLE OF TIPOSAMONESTACIONESYSANCIONES%ROWTYPE INDEX BY BINARY_INTEGER ;
TYPE AMOYSANXALUM_TABLA IS TABLE OF AMOYSANXALUMNO%ROWTYPE INDEX BY BINARY_INTEGER ;

PROCEDURE p_listadoAmonestaciones (
LIST_AMO OUT AMO_TABLA,
RSP OUT varchar2
);

PROCEDURE p_listadoAlumnosXCurso (
LIST_ALUM OUT ALUMXCUR_TABLA,
RSP OUT varchar2,
id_curso IN number DEFAULT 0
);

PROCEDURE p_listadoTiposAmoYSan (
LIST_AMOSAN OUT AMOYSAN_TABLA,
RSP OUT varchar2
);

PROCEDURE p_listadoAmoYSanXAlumno (
LIST_AMOSANXALUM OUT AMOYSANXALUM_TABLA,
RSP OUT varchar2,
ID_ALUMNO IN number DEFAULT 0
);

END;
/


CREATE OR REPLACE PACKAGE BODY LISTADOS AS 
PROCEDURE p_listadoAmonestaciones (
LIST_AMO OUT AMO_TABLA,
RSP OUT varchar2
) IS
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_listadoAmonestaciones';

BEGIN
 	params := '';
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	DBMS_OUTPUT.PUT_LINE('-------------------- Informe Amonestaciones ' || TO_CHAR(f_ini, 'DD/MM/YYYY HH24:MI:SS') || ' -----------------------');
	DBMS_OUTPUT.NEW_LINE;
	
   SELECT *
   BULK COLLECT INTO LIST_AMO
    	      FROM LISTADOAMONESTACIONES;				  

	DBMS_OUTPUT.PUT(rpad('TIPO DE AMONESTACIN',30));
	DBMS_OUTPUT.PUT(rpad('ALUMNO',20));
	DBMS_OUTPUT.PUT(rpad('CURSO',20));
	DBMS_OUTPUT.PUT(rpad('ASIGNATURA',20));
	DBMS_OUTPUT.PUT(rpad('PROFESOR',30));
	DBMS_OUTPUT.PUT(rpad('FECHA',30));
	DBMS_OUTPUT.PUT('GRAVEDAD ');
	DBMS_OUTPUT.PUT_LINE(' COMUNICADO');
	DBMS_OUTPUT.NEW_LINE;

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
		DBMS_OUTPUT.PUT_LINE('NO SE ENCONTRARON DATOS');
		
		RSP := 'ERROR: No existen amonestaciones';

		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al consultar amonestaciones: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END p_listadoAmonestaciones;

PROCEDURE  p_listadoAlumnosXCurso (
LIST_ALUM OUT ALUMXCUR_TABLA,
RSP OUT varchar2,
ID_CURSO IN number DEFAULT 0
) IS
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_listadoAlumnosXCurso';

BEGIN
 	params := '';
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	DBMS_OUTPUT.PUT_LINE('------------------ Informe Alumnos por Curso ' || TO_CHAR(f_ini, 'DD/MM/YYYY HH24:MI:SS') || ' ---------------------');
   
   IF (ID_CURSO IS NULL) OR (ID_CURSO = 0) THEN
      SELECT *
   	  BULK COLLECT INTO LIST_ALUM
    	      FROM ALUMNOSXCURSO;

			  	DBMS_OUTPUT.PUT_LINE('------------------ Todos los cursos ---------------------');  
				DBMS_OUTPUT.NEW_LINE;
	ELSE				  
      SELECT *
   	  BULK COLLECT INTO LIST_ALUM
    	      FROM ALUMNOSXCURSO
			  WHERE IDCURSO = id_curso;
			  
		  	  
			  DBMS_OUTPUT.PUT_LINE('------------------ Curso ' || id_curso || ' ---------------------');
			  	DBMS_OUTPUT.NEW_LINE;
	END IF;
	
	DBMS_OUTPUT.PUT(rpad('CURSO',20));
	DBMS_OUTPUT.PUT(rpad('ALUMNO',30));
	DBMS_OUTPUT.PUT(rpad('DNI',10));
	DBMS_OUTPUT.NEW_LINE;

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
		DBMS_OUTPUT.PUT_LINE('NO SE ENCONTRARON DATOS');
		
		RSP := 'ERROR: No existen alumnos para el curso';

		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al consultar alumnos: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END p_listadoAlumnosXCurso;

PROCEDURE p_listadoTiposAmoYSan (
LIST_AMOSAN OUT AMOYSAN_TABLA,
RSP OUT varchar2
) IS
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_listadoTiposAmoYSan';

BEGIN
 	params := '';
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	DBMS_OUTPUT.PUT_LINE('---------------- Informe Tipos Amonestaciones y Sanciones ' || TO_CHAR(f_ini, 'DD/MM/YYYY HH24:MI:SS') || ' -------------------');
	DBMS_OUTPUT.NEW_LINE;
   
    SELECT *
    BULK COLLECT INTO LIST_AMOSAN
    	      FROM TIPOSAMONESTACIONESYSANCIONES;
	
	
	DBMS_OUTPUT.PUT(rpad('TIPO',30));
	DBMS_OUTPUT.PUT(rpad('DESCRIPCIN',30));
	DBMS_OUTPUT.PUT(rpad('NOMBRE PROFESOR',30));
	DBMS_OUTPUT.PUT(rpad('AMONESTACIN RELACIONADA',30));
	DBMS_OUTPUT.NEW_LINE;

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
		DBMS_OUTPUT.PUT_LINE('NO SE ENCONTRARON DATOS');
		
		RSP := 'ERROR: No existen tipos de amonestaciones ni sanciones';

		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al consultar los tipos de amonestaciones y sanciones: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END p_listadoTiposAmoYSan;

PROCEDURE p_listadoAmoYSanXAlumno (
LIST_AMOSANXALUM OUT AMOYSANXALUM_TABLA,
RSP OUT varchar2,
ID_ALUMNO number DEFAULT 0
) IS
f_ini DATE;
params VARCHAR2(500);
nomproc CONSTANT VARCHAR2(25) := 'p_listadoAmoYSanXAlumno';

BEGIN
 	params := '';
    SELECT SYSDATE INTO f_ini FROM DUAL;
	
	DBMS_OUTPUT.PUT_LINE('------------ Informe Amonestaciones y Sanciones Por Alumno ' || TO_CHAR(f_ini, 'DD/MM/YYYY HH24:MI:SS') || ' ---------------');
	

    IF (ID_ALUMNO IS NULL) OR (ID_ALUMNO = 0) THEN   
	    SELECT *
		    BULK COLLECT INTO LIST_AMOSANXALUM
    	      FROM AMOYSANXALUMNO;
			  
			  	DBMS_OUTPUT.PUT_LINE('------------ Todos los alumnos ---------------');
			  DBMS_OUTPUT.NEW_LINE;
	ELSE
	    SELECT *
		    BULK COLLECT INTO LIST_AMOSANXALUM
    	      FROM AMOYSANXALUMNO
			  WHERE IDALUMNO = ID_ALUMNO;
			  
			  	DBMS_OUTPUT.PUT_LINE('------------ Alumno ' || id_alumno || ' ---------------');
			  DBMS_OUTPUT.NEW_LINE;
	END IF;	
	
	IF LIST_AMOSANXALUM.COUNT = 0 THEN
	   RAISE NO_DATA_FOUND;
    END IF;
	
	DBMS_OUTPUT.PUT(rpad('CLASE',25));
	DBMS_OUTPUT.PUT(rpad('NOMBRE ALUMNO',30));
	DBMS_OUTPUT.PUT(rpad('NOMBRE PROFESOR',30));
	DBMS_OUTPUT.PUT(rpad('NOMBRE ASIGNATURA',20));
	DBMS_OUTPUT.PUT(rpad('NOMBRE CURSO',20));
	DBMS_OUTPUT.PUT(rpad('FECHA',30));
	DBMS_OUTPUT.PUT(rpad('TIPO',30));
	DBMS_OUTPUT.PUT(rpad('GRAVEDAD',10));
	DBMS_OUTPUT.PUT(rpad('COMUNICADO',10));
	DBMS_OUTPUT.PUT(rpad('AUTOMTICA',10));
	DBMS_OUTPUT.PUT_LINE('RESOLUCIN');
	DBMS_OUTPUT.NEW_LINE;

	RSP := 'OK';
	
	p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);
	
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	
		DBMS_OUTPUT.PUT_LINE('NO SE ENCONTRARON DATOS');
		
		RSP := 'ERROR: No existen amonestaciones ni sanciones para este alumno: ' || id_alumno;

		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);	
	
	
	WHEN OTHERS THEN
	
		RSP := 'ERROR: Error al consultar amonestaciones y sanciones para este alumno: ' || sqlerrm;
		ROLLBACK;
		p_insertaLog(nomproc, f_ini, SYSDATE, params, RSP);

END p_listadoAmoYSanXAlumno;

END LISTADOS;
/

