SET SERVEROUTPUT ON

/*---------------------------------------------------------------------------------------
							TABLA CENTROS
---------------------------------------------------------------------------------------*/

CREATE TABLE CENTROS(IDCENTRO NUMBER(5) NOT NULL,
	   		 		 NOMBRE VARCHAR2(200) NOT NULL,
					 DIRECCION VARCHAR2(200),
					 CONSTRAINT PK_CENTROS PRIMARY KEY(IDCENTRO));

/*---------------------------------------------------------------------------------------
							TABLA CURSOS
---------------------------------------------------------------------------------------*/
				  
CREATE TABLE CURSOS(IDCURSO NUMBER(8) NOT NULL,
	   		 		 NOMBRE VARCHAR2(100) NOT NULL,
					 FECHAINICIO DATE NOT NULL,
					 FECHAFIN DATE NOT NULL,
					 IDCENTRO NUMBER(5),
					 CONSTRAINT PK_CURSOS PRIMARY KEY(IDCURSO),
					 CONSTRAINT FK_CURSOS_CENTROS FOREIGN KEY (IDCENTRO) REFERENCES CENTROS(IDCENTRO),
					 CONSTRAINT CHK_FECHAS CHECK (FECHAFIN > FECHAINICIO));

/*---------------------------------------------------------------------------------------
							TABLA FECHASCURSO
---------------------------------------------------------------------------------------*/
					 
CREATE TABLE FECHASCURSO(IDFECHASCURSO NUMBER(8) NOT NULL,
	   		 			 IDCURSO NUMBER(8) NOT NULL,
	   		 			 FECHA DATE NOT NULL,
						 FESTIVO VARCHAR2(1) DEFAULT 'N',
						 DESC_FESTIVO VARCHAR2(100),
 						 CONSTRAINT PK_FECHASCURSO PRIMARY KEY(IDFECHASCURSO),
                         CONSTRAINT FK_FECHASCURSO_CURSOS FOREIGN KEY (IDCURSO) REFERENCES CURSOS(IDCURSO) ON DELETE CASCADE);

/*---------------------------------------------------------------------------------------
							TABLA ALUMNOS
---------------------------------------------------------------------------------------*/

CREATE TABLE ALUMNOS(IDALUMNO NUMBER(8) NOT NULL,
	   		 		 NOMBRE VARCHAR2(50) NOT NULL,
 	   		 		 APELLIDOS VARCHAR2(50) NOT NULL,
					 DNI VARCHAR2(10) NOT NULL,
					 CONSTRAINT PK_ALUMNOS PRIMARY KEY(IDALUMNO));
					 
/*---------------------------------------------------------------------------------------
							TABLA PROFESORES
---------------------------------------------------------------------------------------*/
					 
CREATE TABLE PROFESORES(IDPROFESOR NUMBER(8) NOT NULL,
	   		 		 NOMBRE VARCHAR2(50) NOT NULL,
 	   		 		 APELLIDOS VARCHAR2(50) NOT NULL,
					 DNI VARCHAR2(10) NOT NULL,
					 CONSTRAINT PK_PROFESORES PRIMARY KEY(IDPROFESOR));
					  
/*---------------------------------------------------------------------------------------
							TABLA ASIGNATURAS
---------------------------------------------------------------------------------------*/

CREATE TABLE ASIGNATURAS(IDASIGNATURA NUMBER(6) NOT NULL,
	   		 			NOMBRE VARCHAR2(100) NOT NULL,
						CREDITOS NUMBER(2),
						CONSTRAINT PK_ASIGNATURAS PRIMARY KEY(IDASIGNATURA));
						
/*---------------------------------------------------------------------------------------
							TABLA HORARIOS
---------------------------------------------------------------------------------------*/

CREATE TABLE HORARIOS(IDHORARIO NUMBER(8), DIA NUMBER(1), HORAINICIO VARCHAR2(5), HORAFIN VARCHAR2(5),
	   		 		  CONSTRAINT PK_HORARIO PRIMARY KEY(IDHORARIO),
					  CONSTRAINT CHK_HORARIO_DIA CHECK (DIA BETWEEN 0 AND 7),
					  CONSTRAINT CHK_HORARIO_HORA CHECK (HORAINICIO < HORAFIN));						

/*---------------------------------------------------------------------------------------
							TABLA ASIGNATURASCURSO
---------------------------------------------------------------------------------------*/

CREATE TABLE ASIGNATURASCURSO(IDCURSO NUMBER(8) NOT NULL,
							  IDASIGNATURA NUMBER(8) NOT NULL,
							  CONSTRAINT PK_ASIGNATURASCURSO PRIMARY KEY(IDCURSO,IDASIGNATURA),
							  CONSTRAINT FK_ASIGCURSO_CURSOS FOREIGN KEY (IDCURSO) REFERENCES CURSOS(IDCURSO) ON DELETE CASCADE,
	    					  CONSTRAINT FK_ASIGCURSO_ASIG FOREIGN KEY (IDASIGNATURA) REFERENCES ASIGNATURAS(IDASIGNATURA) ON DELETE CASCADE);

/*---------------------------------------------------------------------------------------
							TABLA HORARIOASIGNATURAS
---------------------------------------------------------------------------------------*/

CREATE TABLE HORARIOASIGNATURAS(IDCURSO NUMBER(8) NOT NULL,
							   IDASIGNATURA NUMBER(8) NOT NULL,
							   IDHORARIO NUMBER(8) NOT NULL,
							   CONSTRAINT PK_HORARIOASIGNATURAS PRIMARY KEY(IDCURSO, IDASIGNATURA, IDHORARIO),
							   CONSTRAINT FK_HORARIOASIG_ASIGCURSO FOREIGN KEY (IDCURSO, IDASIGNATURA) REFERENCES ASIGNATURASCURSO(IDCURSO,IDASIGNATURA) ON DELETE CASCADE,
							   CONSTRAINT FK_HORARIOASIG_HORARIOS FOREIGN KEY (IDHORARIO) REFERENCES HORARIOS(IDHORARIO));
							   
/*---------------------------------------------------------------------------------------
							TABLA CALENDARIOESCOLAR
---------------------------------------------------------------------------------------*/							   

CREATE TABLE CALENDARIOESCOLAR(IDFECHASCURSO NUMBER(8),
  							   IDASIGNATURA NUMBER(8),
							   HORAINICIO VARCHAR2(5),
							   HORAFIN VARCHAR2(5),
                               CONSTRAINT PK_CALENDARIOESCOLAR PRIMARY KEY(IDFECHASCURSO, IDASIGNATURA, HORAINICIO),
							   CONSTRAINT FK_CALENDARIO_FECHAS FOREIGN KEY (IDFECHASCURSO) REFERENCES FECHASCURSO(IDFECHASCURSO) ON DELETE CASCADE,
							   CONSTRAINT FK_CALENDARIO_ASIG FOREIGN KEY (IDASIGNATURA) REFERENCES ASIGNATURAS(IDASIGNATURA) ON DELETE CASCADE);
							   					  
/*---------------------------------------------------------------------------------------
							TABLA PROFESORESASIGNATURA
---------------------------------------------------------------------------------------*/

CREATE TABLE PROFESORESASIGNATURA(IDPROFESOR NUMBER(8) NOT NULL,
	   		 					  IDASIGNATURA NUMBER(6) NOT NULL,
								  CONSTRAINT PK_PROFESORESASIGNATURA PRIMARY KEY(IDPROFESOR, IDASIGNATURA),
								  CONSTRAINT FK_PROFASIG_PROF FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR) ON DELETE CASCADE,
								  CONSTRAINT FK_PROFASIG_ASIG FOREIGN KEY (IDASIGNATURA) REFERENCES ASIGNATURAS(IDASIGNATURA) ON DELETE CASCADE);

/*---------------------------------------------------------------------------------------
							TABLA PROFESORESCURSO
---------------------------------------------------------------------------------------*/
						
CREATE TABLE PROFESORESCURSO(IDPROFESOR NUMBER(8) NOT NULL,
	   		 					  IDCURSO NUMBER(8) NOT NULL,
								  IDHORARIO NUMBER(8),
								  TIPOHORARIO VARCHAR(1) DEFAULT ' ',
								  CONSTRAINT PK_PROFESORESCURSO PRIMARY KEY(IDPROFESOR, IDCURSO, IDHORARIO, TIPOHORARIO),
								  CONSTRAINT FK_PROFCURSO_PROF FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR) ON DELETE CASCADE,
								  CONSTRAINT FK_PROFCURSO_CURSOS FOREIGN KEY (IDCURSO) REFERENCES CURSOS(IDCURSO) ON DELETE CASCADE,
								  CONSTRAINT FK_PROFCURSO_HORARIOS FOREIGN KEY (IDHORARIO) REFERENCES HORARIOS(IDHORARIO),
								  CONSTRAINT CHK_TIPOHORARIO CHECK (TIPOHORARIO IN('P', 'A', ' ')));						

/*---------------------------------------------------------------------------------------
							TABLA ALUMNOSCURSO
---------------------------------------------------------------------------------------*/

CREATE TABLE ALUMNOSCURSO(IDALUMNO NUMBER(8) NOT NULL,
	 					  IDCURSO NUMBER(8) NOT NULL,
					  	  CONSTRAINT PK_ALUMNOSCURSO PRIMARY KEY(IDALUMNO, IDCURSO),
					  	  CONSTRAINT FK_ALUMCURSO_ALUM FOREIGN KEY (IDALUMNO) REFERENCES ALUMNOS(IDALUMNO) ON DELETE CASCADE,
  					  	  CONSTRAINT FK_ALUMCURSO_CURSOS FOREIGN KEY (IDCURSO) REFERENCES CURSOS(IDCURSO) ON DELETE CASCADE);
					  
/*---------------------------------------------------------------------------------------
							TABLA TIPOAMONESTACIONES
---------------------------------------------------------------------------------------*/

CREATE TABLE TIPOAMONESTACIONES(IDTIPOAMO NUMBER(4) NOT NULL,
	   		 					IDPROFESOR NUMBER(8),
	   		 					DESCRIPCION VARCHAR2(100) NOT NULL,
								CONSTRAINT PK_TIPOAMONESTACIONES PRIMARY KEY(IDTIPOAMO),
         					    CONSTRAINT FK_TIPOAMO_PROF FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR));

/*---------------------------------------------------------------------------------------
							TABLA TIPOSANCIONES
---------------------------------------------------------------------------------------*/

CREATE TABLE TIPOSANCIONES(IDTIPOSAN NUMBER(4) NOT NULL,
	   		 				    IDPROFESOR NUMBER(8),
	   		 					DESCRIPCION VARCHAR2(100) NOT NULL,
								IDTIPOAMORELACIONADA NUMBER(4),
								CONSTRAINT PK_TIPOSANCIONES PRIMARY KEY(IDTIPOSAN),
								CONSTRAINT FK_TIPOSAN_PROF FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR),
         					    CONSTRAINT FK_TIPOSAN_TIPOAMO FOREIGN KEY (IDTIPOAMORELACIONADA) REFERENCES TIPOAMONESTACIONES(IDTIPOAMO));

/*---------------------------------------------------------------------------------------
							TABLA AMONESTACIONES
---------------------------------------------------------------------------------------*/

CREATE TABLE AMONESTACIONES(IDAMONESTACION NUMBER(8) NOT NULL,
	   		 				IDALUMNO NUMBER(8) NOT NULL,
	 					  IDCURSO NUMBER(8) NOT NULL,
						  IDASIGNATURA NUMBER(6) NOT NULL,
						  IDPROFESOR NUMBER(8) NOT NULL,
						  FECHA DATE,
						  GRAVEDAD VARCHAR2(1),
						  IDTIPO NUMBER(4) NOT NULL,
						  COMUNICADO VARCHAR2(1),
					      CONSTRAINT PK_AMONESTACIONES PRIMARY KEY(IDAMONESTACION),
					  	  CONSTRAINT FK_AMO_ALUM FOREIGN KEY (IDALUMNO) REFERENCES ALUMNOS(IDALUMNO),
  					  	  CONSTRAINT FK_AMO_ASIGCURSO FOREIGN KEY (IDCURSO,IDASIGNATURA) REFERENCES ASIGNATURASCURSO(IDCURSO,IDASIGNATURA),
   					  	  CONSTRAINT FK_AMO_PROF FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR),
   					  	  CONSTRAINT FK_AMO_TIPOAMO FOREIGN KEY (IDTIPO) REFERENCES TIPOAMONESTACIONES(IDTIPOAMO),
					  	  CONSTRAINT CHK_AMO_GRAVEDAD CHECK (GRAVEDAD IN ('A', 'B', 'M')),
					  	  CONSTRAINT CHK_AMO_COM CHECK (COMUNICADO IN ('S', 'N')));

/*---------------------------------------------------------------------------------------
							TABLA REGLASSANCIONES
---------------------------------------------------------------------------------------*/

CREATE TABLE REGLASSANCIONES(IDREGLA NUMBER(4) NOT NULL,
	   		 				 IDTIPOAMO NUMBER(4) NOT NULL,
							 IDTIPOSAN NUMBER(4) NOT NULL,
							 OPERANDO VARCHAR(2) NOT NULL,
							 VALORNUMERICO NUMBER(4) NOT NULL,
							 CONSTRAINT PK_REGLASSANCIONES PRIMARY KEY(IDREGLA),
							 CONSTRAINT FK_REGLAS_TIPOAMO FOREIGN KEY (IDTIPOAMO) REFERENCES TIPOAMONESTACIONES(IDTIPOAMO),
 							 CONSTRAINT FK_REGLAS_TIPOSAN FOREIGN KEY (IDTIPOSAN) REFERENCES TIPOSANCIONES(IDTIPOSAN),
							 CONSTRAINT CHK_REGLAS CHECK (OPERANDO IN ('>', '<', '=', '>=', '<=', '=<', '=>')));
							 
/*---------------------------------------------------------------------------------------
							TABLA SANCIONES
---------------------------------------------------------------------------------------*/

CREATE TABLE SANCIONES(IDSANCION NUMBER(4) NOT NULL,
	   	  		  				  IDTIPOSAN NUMBER(4) NOT NULL,
								  IDALUMNO NUMBER(8) NOT NULL,
								  IDCURSO NUMBER(8) NOT NULL,
  								  IDPROFESOR NUMBER(8),
								  IDREGLAAPLICADA NUMBER(4),
								  FECHA DATE,
								  RESOLUCION VARCHAR2(200),
	  							  CONSTRAINT PK_SANCIONES PRIMARY KEY(IDSANCION),
								  CONSTRAINT FK_SAN_ALUMNOCURSO FOREIGN KEY (IDALUMNO, IDCURSO) REFERENCES ALUMNOSCURSO(IDALUMNO, IDCURSO),
 								  CONSTRAINT FK_SAN_PROF FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR),
								  CONSTRAINT FK_SAN_TIPOSAN FOREIGN KEY (IDTIPOSAN) REFERENCES TIPOSANCIONES(IDTIPOSAN),
 								  CONSTRAINT FK_SAN_REGLA FOREIGN KEY (IDREGLAAPLICADA) REFERENCES REGLASSANCIONES(IDREGLA));

/*---------------------------------------------------------------------------------------
							TABLA LOGS
---------------------------------------------------------------------------------------*/
								  
CREATE TABLE LOGS(NOMBREPROC VARCHAR2(100),
	   		      FECHAINICIO DATE,
				  FECHAFIN DATE,
				  PARAMSENTRADA VARCHAR2(300),
				  PARAMSSALIDA VARCHAR2(300),
				  RESULTADO VARCHAR2(2));


--Tablas mdulo estadstico

/*---------------------------------------------------------------------------------------
							TABLA EST_AMOXALUMNO
---------------------------------------------------------------------------------------*/

CREATE TABLE EST_AMOXALUMNO(NUMAMO NUMBER(3), IDALUMNO NUMBER(8), 
								  CONSTRAINT FK_EST_AMOXALUMNO_ALUMNOS FOREIGN KEY (IDALUMNO) REFERENCES ALUMNOS(IDALUMNO));
								  
/*---------------------------------------------------------------------------------------
							TABLA EST_SANXALUMNOXANYO
---------------------------------------------------------------------------------------*/
								  
CREATE TABLE EST_SANXALUMNOXANYO(NUMSAN NUMBER(3), IDALUMNO NUMBER(8), ANYO NUMBER(4),
							CONSTRAINT FK_EST_SANXALUM_ALUMNOS FOREIGN KEY (IDALUMNO) REFERENCES ALUMNOS(IDALUMNO));

/*---------------------------------------------------------------------------------------
							TABLA EST_SANXANYOXCURSO
---------------------------------------------------------------------------------------*/

CREATE TABLE EST_SANXANYOXCURSO(NUMSAN NUMBER(3), IDCURSO NUMBER(8), ANYO NUMBER(4),
							CONSTRAINT FK_EST_SANXANYO_CURSOS FOREIGN KEY (IDCURSO) REFERENCES CURSOS(IDCURSO));
							
							
/*---------------------------------------------------------------------------------------
							TABLA EST_ALUMNOMASSAN
---------------------------------------------------------------------------------------*/

CREATE TABLE EST_ALUMNOMASSAN(NUMSAN NUMBER(3), IDALUMNO NUMBER(8), NOMBRE VARCHAR2(50), ANYO NUMBER(4),
								  CONSTRAINT FK_EST_ALUMNOMASSAN_ALUMNOS FOREIGN KEY (IDALUMNO) REFERENCES ALUMNOS(IDALUMNO));
								  
/*---------------------------------------------------------------------------------------
							TABLA EST_PROFESORMASAMO
---------------------------------------------------------------------------------------*/

CREATE TABLE EST_PROFESORMASAMO(NUMAMO NUMBER(3), IDPROFESOR NUMBER(8), NOMBRE VARCHAR2(50), IDCURSO NUMBER(8),
								  CONSTRAINT FK_EST_PROFESORMASAMO_PROF FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR),
								  CONSTRAINT FK_EST_PROFESORMASAMO_CURSOS FOREIGN KEY (IDCURSO) REFERENCES CURSOS(IDCURSO));								  							
								  
/*---------------------------------------------------------------------------------------
							TABLA EST_MEDIASANALUMNOSXCURSO
---------------------------------------------------------------------------------------*/

CREATE TABLE EST_MEDIASANALUMNOSXCURSO(IDALUMNO NUMBER(8), MEDIASAN NUMBER(5,2),
								  CONSTRAINT FK_EST_MEDIASAN_ALUMNOS FOREIGN KEY (IDALUMNO) REFERENCES ALUMNOS(IDALUMNO));
								  
/*---------------------------------------------------------------------------------------
							TABLA EST_MEDIAAMOPROFESORESXANYO
---------------------------------------------------------------------------------------*/

CREATE TABLE EST_MEDIAAMOPROFESORESXANYO(IDPROFESOR NUMBER(8), MEDIAAMO NUMBER(5,2),
								  CONSTRAINT FK_EST_MEDIAAMO_ALUMNOS FOREIGN KEY (IDPROFESOR) REFERENCES PROFESORES(IDPROFESOR));
								  
								  
/*---------------------------------------------------------------------------------------
							TABLA EST_ALUMNOSSINAMO
---------------------------------------------------------------------------------------*/

CREATE TABLE EST_ALUMNOSSINAMO(NUMALUMSINAMO NUMBER(8));		
								  

--Vistas para consultas 

/*---------------------------------------------------------------------------------------
							VISTA LISTADOAMONESTACIONES
---------------------------------------------------------------------------------------*/

CREATE OR REPLACE VIEW LISTADOAMONESTACIONES AS SELECT TA.DESCRIPCION DESCR, AL.NOMBRE || ' ' || AL.APELLIDOS NOMAL, CU.NOMBRE NOMCUR, AI.NOMBRE NOMASIG, PR.NOMBRE || ' ' || PR.APELLIDOS NOMPRO, FECHA F_AMO, GRAVEDAD GRAV, COMUNICADO COM
     	      FROM AMONESTACIONES AM, ALUMNOS AL, CURSOS CU, ASIGNATURAS AI, PROFESORES PR, TIPOAMONESTACIONES TA 
		      WHERE TA.IDTIPOAMO = AM.IDTIPO AND
		   		 AL.IDALUMNO = AM.IDALUMNO AND
				 CU.IDCURSO = AM.IDCURSO AND
				 AI.IDASIGNATURA = AM.IDASIGNATURA AND
				 PR.IDPROFESOR = AM.IDPROFESOR
				 ORDER BY AL.APELLIDOS, FECHA;

/*---------------------------------------------------------------------------------------
							VISTA ALUMNOSXCURSO
---------------------------------------------------------------------------------------*/				 
CREATE OR REPLACE VIEW ALUMNOSXCURSO AS SELECT  AC.IDCURSO, C.NOMBRE NOMCURSO, A.NOMBRE || ' ' || APELLIDOS NOMALUMNO, DNI 
FROM ALUMNOS A, CURSOS C, ALUMNOSCURSO AC 
WHERE C.IDCURSO = AC.IDCURSO AND A.IDALUMNO = AC.IDALUMNO ORDER BY 1;

/*---------------------------------------------------------------------------------------
							VISTA TIPOSAMONESTACIONESYSANCIONES
---------------------------------------------------------------------------------------*/

CREATE OR REPLACE VIEW TIPOSAMONESTACIONESYSANCIONES AS SELECT 'AMONESTACIN' TIPO, P.NOMBRE || ' ' || P.APELLIDOS NOMBRE, DESCRIPCION, 'N/A' AMORELACIONADA
        FROM TIPOAMONESTACIONES TA, PROFESORES P 
		WHERE TA.IDPROFESOR = P.IDPROFESOR
UNION
SELECT 'SANCIN', P.NOMBRE || ' ' || P.APELLIDOS, TS.DESCRIPCION, TAMO.DESCRIPCION
	   FROM TIPOSANCIONES TS, PROFESORES P, TIPOAMONESTACIONES TAMO
	   WHERE TS.IDPROFESOR = P.IDPROFESOR AND IDTIPOAMORELACIONADA = IDTIPOAMO;
	   
/*---------------------------------------------------------------------------------------
							VISTA AMOYSANXALUMNO
---------------------------------------------------------------------------------------*/
	   
CREATE OR REPLACE VIEW AMOYSANXALUMNO AS SELECT 'AMONESTACIN' CLASE, AM.IDALUMNO IDALUMNO, AL.NOMBRE || ' ' || AL.APELLIDOS NOMALUMNO , PR.NOMBRE || ' ' || PR.APELLIDOS NOMPROFESOR ,
	   		  AI.NOMBRE NOMASIGNATURA, CU.NOMBRE NOMCURSO, AM.FECHA FECHA, TA.DESCRIPCION TIPO, DECODE(AM.GRAVEDAD, 'A', 'Alta', 'B', 'Baja', 'M', 'Media','') GRAVEDAD,
			  AM.COMUNICADO COMUNICADO, 'N/A' AUTOMATICA, 'N/A' RESOLUCION 
	   		  FROM AMONESTACIONES AM, ALUMNOS AL, PROFESORES PR, ASIGNATURAS AI, CURSOS CU, TIPOAMONESTACIONES TA
			  WHERE AM.IDALUMNO = AL.IDALUMNO
			  AND AM.IDCURSO = CU.IDCURSO
			  AND AM.IDPROFESOR = PR.IDPROFESOR
			  AND AM.IDASIGNATURA = AI.IDASIGNATURA
			  AND AM.IDTIPO = TA.IDTIPOAMO
	   UNION
	   SELECT 'SANCIN', SA.IDALUMNO, AL.NOMBRE || ' ' || AL.APELLIDOS , PR.NOMBRE || ' ' || PR.APELLIDOS,
	   		  'N/A', CU.NOMBRE, SA.FECHA, TS.DESCRIPCION, 'N/A',
			  'N/A', DECODE(SA.IDREGLAAPLICADA, NULL, 'No', 0, 'No','S'), SA.RESOLUCION  
	   		  FROM SANCIONES SA, ALUMNOS AL, PROFESORES PR, CURSOS CU, TIPOSANCIONES TS
  			  WHERE SA.IDALUMNO = AL.IDALUMNO
			  AND SA.IDCURSO = CU.IDCURSO
			  AND SA.IDPROFESOR = PR.IDPROFESOR
			  AND SA.IDTIPOSAN = TS.IDTIPOSAN  ORDER BY 2;


-- Secuencias

CREATE SEQUENCE CENTROS_SEQ;
CREATE SEQUENCE CURSOS_SEQ;
CREATE SEQUENCE HORARIOS_SEQ;
CREATE SEQUENCE FECHASCURSO_SEQ;
CREATE SEQUENCE ALUMNOS_SEQ;
CREATE SEQUENCE PROFESORES_SEQ;
CREATE SEQUENCE ASIGNATURAS_SEQ;
CREATE SEQUENCE TIPOAMO_SEQ;
CREATE SEQUENCE TIPOSAN_SEQ;
CREATE SEQUENCE AMONESTACIONES_SEQ;
CREATE SEQUENCE REGLAS_SEQ;
CREATE SEQUENCE SANCIONES_SEQ;

--Triggers

CREATE OR REPLACE TRIGGER NUEVOCENTRO 
BEFORE INSERT ON CENTROS FOR EACH ROW
BEGIN
IF :NEW.IDCENTRO is null then
SELECT CENTROS_SEQ.NEXTVAL INTO :NEW.IDCENTRO FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVOHORARIO 
BEFORE INSERT ON HORARIOS FOR EACH ROW
BEGIN
IF :NEW.IDHORARIO is null then
SELECT HORARIOS_SEQ.NEXTVAL INTO :NEW.IDHORARIO FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVOCURSO 
BEFORE INSERT ON CURSOS FOR EACH ROW
BEGIN
IF :NEW.IDCURSO is null then
SELECT CURSOS_SEQ.NEXTVAL INTO :NEW.IDCURSO FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVAFECHACURSO 
BEFORE INSERT ON FECHASCURSO FOR EACH ROW
BEGIN
IF :NEW.IDFECHASCURSO is null then
SELECT FECHASCURSO_SEQ.NEXTVAL INTO :NEW.IDFECHASCURSO FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER FECHASCURSO 
AFTER INSERT ON CURSOS FOR EACH ROW
DECLARE
f_aux DATE;
BEGIN
f_aux := :NEW.FECHAINICIO;
	--Creamos el calendario escolar para este curso y lo iremos actualizando cada vez que se inserte un horario de asignatura o un festivo.
	LOOP
		INSERT INTO FECHASCURSO(IDCURSO, FECHA) VALUES (:NEW.IDCURSO, f_aux);
		f_aux := f_aux + 1;
	EXIT WHEN f_aux > :NEW.FECHAFIN;
	END LOOP; 
END;
/


CREATE OR REPLACE TRIGGER MODFECHASCURSOINICIO 
AFTER UPDATE OF FECHAINICIO ON CURSOS FOR EACH ROW
DECLARE
f_aux DATE;
BEGIN
IF :NEW.FECHAINICIO < :OLD.FECHAINICIO THEN
   f_aux := :NEW.FECHAINICIO;
	LOOP
		INSERT INTO FECHASCURSO(IDCURSO, FECHA) VALUES (:NEW.IDCURSO, f_aux);
		f_aux := f_aux + 1;
	EXIT WHEN f_aux = :OLD.FECHAINICIO;
	END LOOP;
ELSIF :NEW.FECHAINICIO > :OLD.FECHAINICIO THEN
	DELETE FROM FECHASCURSO WHERE FECHA BETWEEN :OLD.FECHAINICIO AND :NEW.FECHAINICIO - 1; 
END IF;
END;
/

CREATE OR REPLACE TRIGGER MODFECHASCURSOFIN 
AFTER UPDATE OF FECHAFIN ON CURSOS FOR EACH ROW
DECLARE
f_aux DATE;
BEGIN
IF :NEW.FECHAFIN > :OLD.FECHAFIN THEN
    f_aux := :OLD.FECHAFIN + 1;
	LOOP
		INSERT INTO FECHASCURSO(IDCURSO, FECHA) VALUES (:NEW.IDCURSO, f_aux);
		f_aux := f_aux + 1;
	EXIT WHEN f_aux > :NEW.FECHAFIN;
	END LOOP;
ELSIF :NEW.FECHAFIN < :OLD.FECHAFIN THEN
	DELETE FROM FECHASCURSO WHERE FECHA BETWEEN :NEW.FECHAFIN + 1 AND :OLD.FECHAFIN; 
END IF;
END;
/


CREATE OR REPLACE TRIGGER NUEVOALUMNO 
BEFORE INSERT ON ALUMNOS FOR EACH ROW
BEGIN
IF :NEW.IDALUMNO is null then
SELECT ALUMNOS_SEQ.NEXTVAL INTO :NEW.IDALUMNO FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVOPROFESOR 
BEFORE INSERT ON PROFESORES FOR EACH ROW
BEGIN
IF :NEW.IDPROFESOR is null then
SELECT PROFESORES_SEQ.NEXTVAL INTO :NEW.IDPROFESOR FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVAASIGNATURA 
BEFORE INSERT ON ASIGNATURAS FOR EACH ROW
BEGIN
IF :NEW.IDASIGNATURA is null then
SELECT ASIGNATURAS_SEQ.NEXTVAL INTO :NEW.IDASIGNATURA FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER ACTUALIZACALENDARIO 
AFTER INSERT OR UPDATE ON HORARIOASIGNATURAS FOR EACH ROW
DECLARE
id_fechascurso NUMBER(8);
new_diasemana NUMBER(1);
new_horainicio VARCHAR2(5);
new_horafin VARCHAR2(5);

BEGIN

SELECT DIA, HORAINICIO, HORAFIN
INTO new_diasemana, new_horainicio, new_horafin
FROM HORARIOS
WHERE IDHORARIO = :NEW.IDHORARIO;

INSERT INTO CALENDARIOESCOLAR(IDFECHASCURSO, IDASIGNATURA, HORAINICIO, HORAFIN)
	   		(SELECT IDFECHASCURSO, :NEW.IDASIGNATURA, new_horainicio, new_horafin
             FROM FECHASCURSO 
  			 WHERE IDCURSO = :NEW.IDCURSO AND
 			 new_diasemana = TO_NUMBER(TO_CHAR(FECHA, 'D')));
EXCEPTION
WHEN DUP_VAL_ON_INDEX THEN

	  UPDATE CALENDARIOESCOLAR SET HORAINICIO = new_horainicio, HORAFIN = new_horafin
   WHERE HORAINICIO = new_horainicio AND
   		 IDASIGNATURA = :NEW.IDASIGNATURA AND 
   		 IDFECHASCURSO IN (SELECT IDFECHASCURSO
             FROM FECHASCURSO 
  			 WHERE IDCURSO = :NEW.IDCURSO AND
 			 new_diasemana = TO_NUMBER(TO_CHAR(FECHA, 'D')));
END;
/

CREATE OR REPLACE TRIGGER BORRACALENDARIO 
AFTER DELETE ON HORARIOASIGNATURAS FOR EACH ROW
DECLARE
id_fechascurso NUMBER(8);
old_diasemana NUMBER(1);
old_horainicio VARCHAR2(5);

BEGIN
	 
	 SELECT DIA, HORAINICIO
	 INTO old_diasemana, old_horainicio
	 FROM HORARIOS
	 WHERE IDHORARIO = :OLD.IDHORARIO;

	 DELETE CALENDARIOESCOLAR WHERE IDASIGNATURA = :OLD.IDASIGNATURA AND
									 HORAINICIO = old_horainicio AND
									 IDFECHASCURSO IN (SELECT IDFECHASCURSO 
									 			      FROM FECHASCURSO 
													  WHERE IDCURSO = :OLD.IDCURSO AND
													  old_diasemana = TO_NUMBER(TO_CHAR(FECHA, 'D'))); 
END;
/


CREATE OR REPLACE TRIGGER NUEVATIPOAMO 
BEFORE INSERT ON TIPOAMONESTACIONES FOR EACH ROW
BEGIN
IF :NEW.IDTIPOAMO is null then
SELECT TIPOAMO_SEQ.NEXTVAL INTO :NEW.IDTIPOAMO FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVATIPOSAN 
BEFORE INSERT ON TIPOSANCIONES FOR EACH ROW
BEGIN
IF :NEW.IDTIPOSAN is null then
SELECT TIPOSAN_SEQ.NEXTVAL INTO :NEW.IDTIPOSAN FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVAAMONESTACION 
BEFORE INSERT ON AMONESTACIONES FOR EACH ROW
BEGIN
IF :NEW.IDAMONESTACION is null then
SELECT AMONESTACIONES_SEQ.NEXTVAL INTO :NEW.IDAMONESTACION FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVAREGLA 
BEFORE INSERT ON REGLASSANCIONES FOR EACH ROW
BEGIN
IF :NEW.IDREGLA is null then
SELECT REGLAS_SEQ.NEXTVAL INTO :NEW.IDREGLA FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER NUEVASANCION 
BEFORE INSERT ON SANCIONES FOR EACH ROW
BEGIN
IF :NEW.IDSANCION is null then
SELECT SANCIONES_SEQ.NEXTVAL INTO :NEW.IDSANCION FROM DUAL;
END IF;
END;
/	

CREATE OR REPLACE TRIGGER COMPROBARREGLAS 
AFTER INSERT OR UPDATE ON AMONESTACIONES FOR EACH ROW
DECLARE
RSP VARCHAR2(700);
PRAGMA autonomous_transaction;
BEGIN
p_comprobarReglas(:NEW.IDTIPO, :NEW.IDALUMNO, :NEW.IDCURSO, RSP);
END;
/		

INSERT INTO HORARIOS VALUES (0,0,'','');

COMMIT;